import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Popover, Button } from 'antd'
import { EditOutlined, ToolOutlined, DeleteOutlined, FontColorsOutlined, PrinterOutlined } from '@ant-design/icons'

import MKEmitter from '@/utils/events.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncIconComponent from '@/utils/asyncIconComponent'
import { resetStyle } from '@/utils/utils-custom.js'
import getSettingForm from './options'
import './index.scss'

const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const CopyComponent = asyncIconComponent(() => import('@/menu/components/share/copycomponent'))
const PasteComponent = asyncIconComponent(() => import('../paste'))
const GroupComponents = asyncComponent(() => import('../groupcomponents'))

class NormalGroup extends Component {
  static propTpyes = {
    group: PropTypes.object,
    deletecomponent: PropTypes.func,
    updateConfig: PropTypes.func,
  }

  state = {
    group: null,
    editab: null,
  }

  UNSAFE_componentWillMount () {
    const { group } = this.props

    if (group.isNew) {
      let _group = {
        uuid: group.uuid,
        type: group.type,
        tabId: group.tabId || '',
        parentId: group.parentId || '',
        subtype: group.subtype,
        width: 24,
        name: group.name,
        setting: { width: 24, name: group.name },
        style: { marginLeft: '8px', marginRight: '8px', marginTop: '8px', marginBottom: '8px' },
        components: []
      }
      this.setState({
        group: _group
      })
      this.props.updateConfig(_group)
    } else {
      this.setState({
        group: fromJS(group).toJS()
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    MKEmitter.addListener('tabsChange', this.handleTabsChange)
    MKEmitter.addListener('submitComponentStyle', this.updateComponentStyle)
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('tabsChange', this.handleTabsChange)
    MKEmitter.removeListener('submitComponentStyle', this.updateComponentStyle)
  }

  updateComponentStyle = (parentId, keys, style) => {
    const { group } = this.state

    if (group.uuid !== parentId) return

    let components = group.components.map(item => {
      if (keys.includes(item.uuid)) {
        item.style = {...item.style, ...style}
      }
      return item
    })

    this.setState({
      group: {...group, components: []}
    }, () => {
      this.updateComponent({...group, components: components})
    })
  }

  changeStyle = () => {
    const { group } = this.state

    MKEmitter.emit('changeStyle', ['background', 'border', 'padding', 'margin', 'shadow'], group.style, this.getStyle)
  }

  getStyle = (style) => {
    let _card = {...this.state.group, style}

    this.setState({
      group: _card
    })
    
    this.props.updateConfig(_card)
  }

  handleTabsChange = (parentId) => {
    const { group } = this.state

    if (parentId === group.parentId) {
      MKEmitter.emit('tabsChange', group.uuid)
    }
  }

  updateComponent = (component) => {
    const { group } = this.state

    if (!is(fromJS(group.setting), fromJS(component.setting)) || !is(fromJS(group.style), fromJS(component.style))) {
      // 注册事件-标签变化，通知组内元素
      MKEmitter.emit('tabsChange', group.uuid)
    }

    component.width = component.setting.width
    component.name = component.setting.name

    this.setState({
      group: component
    })
    this.props.updateConfig(component)
  }

  insert = (item) => {
    let group = fromJS(this.state.group).toJS()

    item.parentId = group.parentId

    group.components.push(item)

    this.setState({group})
    this.props.updateConfig(group)
  }

  getWrapForms = () => {
    const { setting } = this.state.group

    return getSettingForm(setting)
  }

  updateWrap = (res) => {
    this.updateComponent({...this.state.group, setting: res})
  }

  clickComponent = (e) => {
    if (sessionStorage.getItem('style-control') === 'true' || sessionStorage.getItem('style-control') === 'component') {
      e.stopPropagation()
      MKEmitter.emit('clickComponent', this.state.group)
    }
  }

  render() {
    const { group } = this.state
    let _style = resetStyle(group.style)

    let paddingTop = true
    if (group.style.paddingTop && parseInt(group.style.paddingTop) >= 28) {
      paddingTop = false
    }

    return (
      <div className={'menu-group-edit-box ' + (paddingTop ? 'padding ' : '') + (group.setting.layout || '')} style={_style} onClick={this.clickComponent} id={group.uuid}>
        <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
          <div className="mk-popover-control">
            <NormalForm title="分组设置" width={700} update={this.updateWrap} getForms={this.getWrapForms}>
              <EditOutlined style={{color: '#1890ff'}} title="编辑"/>
            </NormalForm>
            <CopyComponent type="tabs" card={group}/>
            <PasteComponent insert={this.insert} />
            <FontColorsOutlined className="style" title="调整样式" onClick={this.changeStyle}/>
            <DeleteOutlined className="close" title="delete" onClick={() => this.props.deletecomponent(group.uuid)} />
          </div>
        } trigger="hover">
          <ToolOutlined />
        </Popover>
        {group.setting && group.setting.print === 'true' ? <Button className="print-button" onClick={this.print}><PrinterOutlined /></Button> : null}
        <GroupComponents config={group} handleList={this.updateComponent} deleteCard={this.deleteCard} />
      </div>
    )
  }
}

export default NormalGroup